/*
  File name        : task.c
  Date of creation : 20/2/2012
  Version          : 1.0
  Copyright        : Soft'n'design
  Author           : Laurent Mazet <mazet@softndesign.org>

  Description      : This file contains tests for asynchronous task thread.

  History          :
  - initial version
*/

/* depend: */
/* cflags: */
/* linker: -lmcore -lpthread -lm */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <snd/task.h>
#include <snd/mcore-verbose.h>

float process_time_us = 10000.0f;

int pseudo_random (unsigned int seed)
{

    int i;

    /* Pseudo random sequence initialisation */
    unsigned int x1 = 1;
    unsigned int x2 = seed;
    unsigned int x3 = 1 - seed;

    /* Pseudo random sequence initial scrambling */
    for (i = 0; i < 1000; i++) {
        unsigned int state;

        state = ((x1 >> 3) ^ x1) & 1;
        x1 = (state << 30) | (x1 >> 1);

        state = ((x2 >> 3) ^ (x2 >> 2) ^ (x2 >> 1) ^ x2) & 1;
        x2 = (state << 30) | (x2 >> 1);

        state = ((x3 >> 4) ^ (x2 >> 2) ^ x2) & 1;
        x3 = (state << 30) | (x2 >> 1);        
    }

    int r = (x1 ^ x2 ^ x3);

    return r;
}

int randn (void)
{

    int i = 0;
    float x, y;

    int r = 0;
    for (i = 0; i < 10; i++) {

        do
            /* generate a signal which is average random in [0,1] */
            x = (float) rand () / ((float)RAND_MAX + 1);

        while (x == 0);     /* x should not be zero */

        /* generate another signal which is average random in [0,1] */
        y = (float) rand () / ((float)RAND_MAX + 1);

        /* generate noise which average is 0, and variance is sigma */
        float z = sqrtf (-2. * logf (x)) * cosf (2 * M_PI * y);

        r = z * (1 << 31);
    }
    
    return r;
}

typedef struct {
    int *buffer;
    int length;
} data_t;

data_t *data = NULL;

int huge_process (sub_task_t *s, int id)
{

    if ((s == NULL) || (id != 0)) return -1;

    int i;

    /* new seed */
    int seed = randn();

    /* main loop */
    while (1) {

        /* fill buffer */
        for (i = 0; i < data->length; i++) {
            data->buffer[i] = pseudo_random (seed);
            seed = data->buffer[i];
        }

        usleep (process_time_us);

        printf ("one loop done\n");
    }

    return 0;
}

int main (int argc, char *argv[])
{

    int nb_loops = 10;
    int length = 10;

    /* process arguments */
    if (argc > 1) {
        printf ("usage: %s\n", argv[0]);
        exit (1);
    }

    /* change verbose level */
    CHANGE_VERBOSE_LEVEL (coretools, 1);

    /* init structure */
    data = (data_t *) calloc (1, sizeof (data_t));
    data->buffer = (int *) calloc (length, sizeof (int));
    data->length = length;

    /* create sync task */
    task_t *process = create_async_task ("process", huge_process, 70, 1, "0+1+2+3");

    usleep (nb_loops * process_time_us);

    /* global statistics */
    task_stat (process, 0);

    /* cleaning */
    free (data->buffer);
    free (data);

    return 0;
}

/* vi:set tabstop=4 expandtab shiftwidth=4: this line set vi mode */
